pub enum JoinStrategy {
Inner,
Left,
Right,
Full,
}Expand description
Strategy for joining query results from different databases
Variants
Inner
Inner join: Only include rows that exist in both datasets
Left
Left join: Include all rows from left dataset, matching rows from right
Right
Right join: Include all rows from right dataset, matching rows from left
Full
Full outer join: Include all rows from both datasets